//Copyright Monogram (Grant & Union) 2021
#include "MonogramCore.h"

#include "UObject/Package.h"
#include "UObject/UObjectBaseUtility.h"
#include "UObject/UObjectGlobals.h"

#define LOCTEXT_NAMESPACE "UMonogramCore"


UMonogramCore::UMonogramCore() : UEngineSubsystem()
{

}

UMonogramCore::~UMonogramCore()
{
	delete monogramThread;
	monogramThread = nullptr;
}

void UMonogramCore::AddReferencedObjects(UObject* InThis, FReferenceCollector& Collector)
{
	UObject::AddReferencedObjects(InThis, Collector);
	auto This = CastChecked<UMonogramCore>(InThis);
	Collector.AddReferencedObjects(This->MonogramActions);
}

void UMonogramCore::Initialize(FSubsystemCollectionBase& Collection)
{
	if (monogramThread)
	{
		UE_LOG(LogTemp, Warning, TEXT("Thread already running. Skipping threatstart."));
	}
	monogramThread = new TCPMonogramThread(this);
	frunnablePointer = FRunnableThread::Create(monogramThread, TEXT("TCP watcher thread"), 0);

	TArray<UClass*> Results;
	GetDerivedClasses(UMonogramAction::StaticClass(), Results, true);
	for (UClass* HandlerClass : Results)
	{
		if (!HandlerClass->HasAnyClassFlags(CLASS_Abstract | CLASS_Deprecated | CLASS_NewerVersionExists))
		{
			auto action = NewObject<UMonogramAction>(GetTransientPackage(), HandlerClass);
			if (action->Name() != FName(TEXT(""))) {
				MonogramActions.Add(action->Name(), action);
			}
		}
	}
}

void UMonogramCore::Deinitialize()
{
	if (monogramThread != nullptr)
	{
		delete monogramThread;
		monogramThread = nullptr;
	}
}


void FMonogramCoreModuleInterface::StartupModule()
{

}

void FMonogramCoreModuleInterface::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
}


#undef LOCTEXT_NAMESPACE
IMPLEMENT_MODULE(FMonogramCoreModuleInterface, MonogramCore)
