//Copyright Monogram (Grant & Union) 2021

#pragma once

#include "Engine.h"
#include "HAL/RunnableThread.h"
#include "HAL/Runnable.h"
#include "CoreMinimal.h"
#include "Containers/UnrealString.h"
#include "Containers/Map.h"
#include "Subsystems/EngineSubsystem.h"

#include "TCPMonogramThread.h"
#include "MonogramAction.h"

#include "MonogramCore.generated.h"

class FMonogramCoreModuleInterface : public IModuleInterface
{	/** IModuleInterface implementation */
	virtual void StartupModule() override;
	virtual void ShutdownModule() override;
};

DECLARE_DYNAMIC_MULTICAST_DELEGATE_FourParams(FMonogramDataReceivedSignature, FName, rawOut, FName, operationOut, FName, inputOut, float, valueOut);

UCLASS()
class MONOGRAMCORE_API UMonogramCore : public UEngineSubsystem, public IModuleInterface
{
	GENERATED_BODY()
public:
	// Statically override this method to add the MonogramActions MultiMap to
	// the garbage collector.
	static void AddReferencedObjects(UObject* InThis, FReferenceCollector& Collector);

	virtual void Initialize(FSubsystemCollectionBase& Collection) override;
	virtual void Deinitialize() override;

	// MonogramDataReceived is the outgoing blueprint event, it has no C++
	// implementation we call it to notify blueprint code when there's a
	// message.
	UPROPERTY(BlueprintAssignable, Category = "MonogramEvent")
	FMonogramDataReceivedSignature MonogramDataReceived;
	
	UMonogramCore();
	~UMonogramCore();


private:
	FRunnableThread* frunnablePointer = nullptr;
	TCPMonogramThread* monogramThread = nullptr;
	TMultiMap<FName, UMonogramAction*> MonogramActions;
	friend class TCPMonogramThread;
};
