//Copyright Monogram (Grant & Union) 2021
#include "Actions/CineCameraAperture.h"
#include "Engine/Selection.h"
#include "CineCameraActor.h"
#include "CineCameraComponent.h"

void UCineCameraAperture::Handle(FName Command, Operation Operation, float Value)
{
	if (!GEditor) return;
	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;

	ACineCameraActor* topSelected = selected->GetTop<ACineCameraActor>();
	if (!topSelected) return;

	UCineCameraComponent* cineComponent = topSelected->GetCineCameraComponent();

	// We are using F = 2 ^ (0.5 * N)
	// N is what we are increasing with the dial

	float newN;
	if (Operation == Operation::SET) {
		newN = Value;
	}
	else
	{
		//Find the old N by 2log2(F), then add the parseFloat for the new N
		newN = (2 * (log2(cineComponent->CurrentAperture))) + Value;
	}
	
	this->PreAction(Cast<AActor>(topSelected));
	FProperty* CameraComponent = FindFieldChecked<FProperty>(ACineCameraActor::StaticClass(), "CameraComponent");
	FProperty* CurrentApatureProperty = FindFieldChecked<FProperty>(UCineCameraComponent::StaticClass(), "CurrentAperture");

	FEditPropertyChain changeChain;
	changeChain.AddHead(CurrentApatureProperty);
	changeChain.AddHead(CameraComponent);

	changeChain.SetActiveMemberPropertyNode(CurrentApatureProperty);
	Cast<UObject>(cineComponent)->PreEditChange(changeChain);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);
	cineComponent->CurrentAperture = (pow(2, (0.5 * newN)));
	this->PostAction(Cast<AActor>(topSelected));
}
