//Copyright Monogram (Grant & Union) 2021
#include "Actions/CineCameraCycleAutoFocus.h"
#include "Engine/Selection.h"
#include "CineCameraActor.h"
#include "CineCameraComponent.h"

void UCineCameraCycleAutoFocus::Handle(FName Command, Operation Operation, float Value)
{
	if (!GEditor) return;
	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;

	ACineCameraActor* topSelected = selected->GetTop<ACineCameraActor>();
	if (!topSelected) return;

	this->PreAction(Cast<AActor>(topSelected));
	UCineCameraComponent* cineComponent = topSelected->GetCineCameraComponent();

	FProperty* CameraComponent = FindFieldChecked<FProperty>(ACineCameraActor::StaticClass(), "CameraComponent");
	FProperty* FocusSettingsProperty = FindFieldChecked<FProperty>(UCineCameraComponent::StaticClass(), "FocusSettings");
	FProperty* FocusMethodProperty = FindFieldChecked<FProperty>(FCameraFocusSettings::StaticStruct(), "FocusMethod");

	FEditPropertyChain changeChain;
	changeChain.AddHead(FocusMethodProperty);//reverse the order here
	changeChain.AddHead(FocusSettingsProperty);
	changeChain.AddHead(CameraComponent);

	changeChain.SetActiveMemberPropertyNode(FocusMethodProperty);
	changeChain.SetActiveMemberPropertyNode(FocusSettingsProperty);
	Cast<UObject>(cineComponent)->PreEditChange(changeChain);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);

	if (cineComponent->FocusSettings.FocusMethod == ECameraFocusMethod::Manual)
	{
		cineComponent->FocusSettings.FocusMethod = ECameraFocusMethod::Tracking;
	}
	else if (cineComponent->FocusSettings.FocusMethod == ECameraFocusMethod::Tracking)
	{
		cineComponent->FocusSettings.FocusMethod = ECameraFocusMethod::Manual;
	}
	else
	{
		cineComponent->FocusSettings.FocusMethod = ECameraFocusMethod::Tracking;
	}
	this->PostAction(Cast<AActor>(topSelected));
}
