//Copyright Monogram (Grant & Union) 2021
#include "Actions/CineCameraFocalLength.h"
#include "Engine/Selection.h"
#include "CineCameraActor.h"
#include "CineCameraComponent.h"

void UCineCameraFocalLength::Handle(FName Command, Operation Operation, float Value)
{
	if (!GEditor) return;
	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;

	ACineCameraActor* topSelected = selected->GetTop<ACineCameraActor>();
	if (!topSelected) return;

	UCineCameraComponent* cineComponent = topSelected->GetCineCameraComponent();

	// We are using f = 2 ^ x
	// x is what we are increasing with the dial

	float newX;
	if (Operation == Operation::SET)
	{
		newX = Value;
	}
	else
	{
		//Find the old x by log2(f), then add the parseFloat for the new x
		newX = (log2(cineComponent->CurrentFocalLength)) + Value;
	}

	this->PreAction(Cast<AActor>(topSelected));
	FProperty* CameraComponent = FindFieldChecked<FProperty>(ACineCameraActor::StaticClass(), "CameraComponent");
	FProperty* CurrentFocalLengthProperty = FindFieldChecked<FProperty>(UCineCameraComponent::StaticClass(), "CurrentFocalLength");

	FEditPropertyChain changeChain;
	changeChain.AddHead(CurrentFocalLengthProperty);//reverse the order here
	changeChain.AddHead(CameraComponent);

	changeChain.SetActiveMemberPropertyNode(CurrentFocalLengthProperty);
	Cast<UObject>(cineComponent)->PreEditChange(changeChain);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);

#if (ENGINE_MAJOR_VERSION == 4) && (ENGINE_MINOR_VERSION < 24)
	cineComponent->CurrentFocalLength = pow(2, newX);
#else
	cineComponent->SetCurrentFocalLength(pow(2, newX));
#endif
	
	this->PostAction(Cast<AActor>(topSelected));
}
