//Copyright Monogram (Grant & Union) 2021
#include "Actions/CineCameraManualFocus.h"
#include "Engine/Selection.h"
#include "CineCameraActor.h"
#include "CineCameraComponent.h"

void UCineCameraManualFocus::Handle(FName Command, Operation Operation, float Value)
{
	if (!GEditor) return;
	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;

	ACineCameraActor* topSelected = selected->GetTop<ACineCameraActor>();
	if (!topSelected) return;

	UCineCameraComponent* cineComponent = topSelected->GetCineCameraComponent();

	//Using the formula D = 2 ^ n
	//Where n is the input from the dial
	//This is the same scaling as focal lenght
	float newN = log2(cineComponent->FocusSettings.ManualFocusDistance) + Value;
	this->PreAction(Cast<AActor>(topSelected));
	FProperty* CameraComponent = FindFieldChecked<FProperty>(ACineCameraActor::StaticClass(), "CameraComponent");
	FProperty* FocusSettingsProperty = FindFieldChecked<FProperty>(UCineCameraComponent::StaticClass(), "FocusSettings");
	FProperty* FocusDistanceProperty = FindFieldChecked<FProperty>(FCameraFocusSettings::StaticStruct(), "ManualFocusDistance");

	FEditPropertyChain changeChain;
	changeChain.AddHead(FocusDistanceProperty);//reverse the order here
	changeChain.AddHead(FocusSettingsProperty);
	changeChain.AddHead(CameraComponent);

	changeChain.SetActiveMemberPropertyNode(FocusDistanceProperty);
	changeChain.SetActiveMemberPropertyNode(FocusSettingsProperty);
	Cast<UObject>(cineComponent)->PreEditChange(changeChain);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);
	cineComponent->FocusSettings.FocusMethod = ECameraFocusMethod::Manual;
	cineComponent->FocusSettings.ManualFocusDistance = pow(2, newN);
	this->PostAction(Cast<AActor>(topSelected));
}
