//Copyright Monogram (Grant & Union) 2021
#include "Actions/LightColor.h"
#include "Engine/Selection.h"
#include "Engine/Light.h"
#include "Components/LightComponent.h"

void ULightColor::Handle(FName Command, Operation Operation, float Value)
{
	ColorComponent::ComponentType component = ColorComponent::none;

	if (Command == TEXT("A"))
	{
		component = ColorComponent::alpha;
	}
	else if (Command == TEXT("R"))
	{
		component = ColorComponent::red;
	}
	else if (Command == TEXT("G"))
	{
		component = ColorComponent::green;
	}
	else if (Command == TEXT("temp"))
	{
		component = ColorComponent::temp;
	}
	else if (Command == TEXT("B"))
	{
		component = ColorComponent::blue;
	}
	else if (Command == TEXT("H"))
	{
		component = ColorComponent::hue;
	}
	else if (Command == TEXT("S"))
	{
		component = ColorComponent::saturation;
	}
	else if (Command == TEXT("V"))
	{
		component = ColorComponent::Value;
	}

	if (component == ColorComponent::none) return;

	if (!GEditor) return;

	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;

	ALight* topSelected = selected->GetTop<ALight>();
	if (!topSelected) return;

	ULightComponent* lightComponent = topSelected->GetLightComponent();

	FLinearColor lightColor = topSelected->GetLightColor();
	if (component == ColorComponent::red)
	{
		if (Operation == Operation::ADJUST)
		{
			lightColor.R += Value;
		}
		else if (Operation == Operation::SET)
		{
			lightColor.R = Value;
		}
	}
	else if (component == ColorComponent::green)
	{
		if (Operation == Operation::ADJUST)
		{
			lightColor.G += Value;
		}
		else if (Operation == Operation::SET)
		{
			lightColor.G = Value;
		}
	}
	else if (component == ColorComponent::blue)
	{
		if (Operation == Operation::ADJUST)
		{
			lightColor.B += Value;
		}
		else if (Operation == Operation::SET)
		{
			lightColor.B = Value;
		}
	}
	else if (component == ColorComponent::temp)
	{
		lightComponent->bUseTemperature = true;
		if (Operation == Operation::ADJUST)
		{
			lightComponent->SetTemperature(lightComponent->Temperature + Value);
		}
		else if (Operation == Operation::SET)
		{
			lightComponent->SetTemperature(Value);
		}
	}
	else if (component == ColorComponent::alpha)
	{
		if (Operation == Operation::ADJUST)
		{
			lightColor.A += Value;
		}
		else if (Operation == Operation::SET)
		{
			lightColor.A = Value;
		}
	}
	else if (component == ColorComponent::hue || component == ColorComponent::saturation || component == ColorComponent::Value)
	{
		FLinearColor HSVColor = lightColor.LinearRGBToHSV();
		if (component == ColorComponent::hue)
		{
			if (Operation == Operation::ADJUST)
			{
				HSVColor.R += (Value * 100.00);
			}
			else if (Operation == Operation::SET)
			{
				HSVColor.R = Value;
			}

			if (HSVColor.R < 0)
			{
				//if it's negative wrap it around.
				HSVColor.R = 360 + HSVColor.R; //R will be negative here
			}
			else if (HSVColor.R > 360)
			{
				HSVColor.R = 0.0 + (HSVColor.R - 360);
			}
		}
		else if (component == ColorComponent::saturation)
		{
			if (Operation == Operation::ADJUST)
			{
				HSVColor.G += Value;
			}
			else if (Operation == Operation::SET)
			{
				HSVColor.G = Value;
			}

			if (HSVColor.G < 0)
			{
				HSVColor.G = 0;
			}
			else if (HSVColor.G > 1.0)
			{
				HSVColor.G = 1.0;
			}
		}
		else if (component == ColorComponent::Value)
		{
			if (Operation == Operation::ADJUST)
			{
				HSVColor.B += Value;
			}
			else if (Operation == Operation::SET)
			{
				HSVColor.B = Value;
			}
		}
		lightColor = HSVColor.HSVToLinearRGB();
	}
	this->PreAction(Cast<AActor>(topSelected));
	FProperty* LightComponentProperty = FindFieldChecked<FProperty>(ALight::StaticClass(), "LightComponent");
	FProperty* LightColourProperty = FindFieldChecked<FProperty>(ULightComponent::StaticClass(), "LightColor");

	FEditPropertyChain changeChain;
	changeChain.AddHead(LightColourProperty);
	changeChain.AddHead(LightComponentProperty);

	changeChain.SetActiveMemberPropertyNode(LightColourProperty);
	Cast<UObject>(lightComponent)->PreEditChange(changeChain);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);

	topSelected->SetLightColor(lightColor);

	FPropertyChangedEvent changeEvent = FPropertyChangedEvent(LightColourProperty);
	Cast<UObject>(topSelected)->PostEditChangeProperty(changeEvent);
	this->PostAction(Cast<AActor>(topSelected));
}
