//Copyright Monogram (Grant & Union) 2021
#include "Actions/LightIntensity.h"
#include "Engine/Selection.h"
#include "Components/LightComponent.h"

void ULightIntensity::Handle(FName Command, Operation Operation, float Value)
{
	if (!GEditor) return;
	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;

	ALight* topSelected = selected->GetTop<ALight>();
	if (!topSelected) return;
	ULightComponent* lightComponent = topSelected->GetLightComponent();
	this->PreAction(Cast<AActor>(topSelected));

	FProperty* LightComponentProperty = FindFieldChecked<FProperty>(ALight::StaticClass(), "LightComponent");
	FProperty* LightIntensityProperty = FindFieldChecked<FProperty>(ULightComponent::StaticClass(), "Intensity");

	FEditPropertyChain changeChain;
	changeChain.AddHead(LightIntensityProperty);
	changeChain.AddHead(LightComponentProperty);

	changeChain.SetActiveMemberPropertyNode(LightIntensityProperty);
	Cast<UObject>(lightComponent)->PreEditChange(changeChain);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);

	if (Operation == Operation::ADJUST)
	{
		lightComponent->Intensity += Value;
	}
	else if (Operation == Operation::SET)
	{
		lightComponent->Intensity = Value;
	}

	FPropertyChangedEvent changeEvent = FPropertyChangedEvent(LightIntensityProperty);
	Cast<UObject>(topSelected)->PostEditChangeProperty(changeEvent);
	this->PostAction(Cast<AActor>(topSelected));
}
