//Copyright Monogram (Grant & Union) 2021

#include "Actions/Move.h"
#include "Engine/Selection.h"

void UMove::Handle(FName Command, Operation Operation, float Value)
{
	FVector delta(0.0f, 0.0f, 0.0f);
	if (Command == TEXT("XLocal") || Command == TEXT("XWorld"))
	{
		delta.X = Value;
	}
	else if (Command == TEXT("YLocal") || Command == TEXT("YWorld"))
	{
		delta.Y = Value;
	}
	else if (Command == TEXT("ZLocal") || Command == TEXT("ZWorld"))
	{
		delta.Z = Value;
	}
	if (!GEditor) return;
	USelection* selected = GEditor->GetSelectedActors();
	if (!selected) return;
	AActor* topSelected = selected->GetTop<AActor>();
	if (!topSelected) return;

	FHitResult hitOut;
	this->PreAction(Cast<AActor>(topSelected));

	FProperty* PositionProperty = FindFieldChecked<FProperty>(USceneComponent::StaticClass(), "RelativeLocation");

	FEditPropertyChain changeChain;
	changeChain.AddHead(PositionProperty);//reverse the order here

	changeChain.SetActiveMemberPropertyNode(PositionProperty);
	Cast<UObject>(topSelected)->PreEditChange(changeChain);
	

	if (Command == TEXT("XLocal") || Command == TEXT("YLocal") || Command == TEXT("ZLocal"))
	{
		topSelected->AddActorLocalOffset(delta);
	}
	else {
		topSelected->AddActorWorldOffset(delta);
	}

	FPropertyChangedEvent changeEvent = FPropertyChangedEvent(PositionProperty);
	Cast<UObject>(topSelected)->PostEditChangeProperty(changeEvent);
	this->PostAction(Cast<AActor>(topSelected));
}
