//Copyright Monogram (Grant & Union) 2021
#include "MonogramEditorActions.h"


#include "Editor.h"
#include "UObject/Package.h"
#include "UObject/UObjectBaseUtility.h"
#include "UObject/UObjectGlobals.h"
#include "Editor/UnrealEdEngine.h"
#include "UnrealEdGlobals.h"
#include "GenericPlatform/GenericPlatformFile.h"

#include "MonogramTransactionAggrigator.h"

#define LOCTEXT_NAMESPACE "UMonogramEditorActions"

void FSequencerCommands::RegisterCommands()
{
}

UMonogramEditorActions::UMonogramEditorActions() : UEngineSubsystem()
{

}

UMonogramEditorActions::~UMonogramEditorActions()
{

}


void UMonogramEditorActions::Initialize(FSubsystemCollectionBase& Collection)
{
	//install monogram integration in the Monogram creator folder.

#if PLATFORM_MAC
	IPlatformFile& platformFiles = FPlatformFileManager::Get().GetPlatformFile();
	FString localPath = FPlatformProcess::UserSettingsDir();
	localPath.RemoveFromEnd(TEXT("Epic/"), ESearchCase::IgnoreCase);
	localPath += "Monogram/Service/integrations/unreal";
	FString folderPath = FPaths::ProjectPluginsDir() + "Monogram/Resources/unreal";
#elif PLATFORM_WINDOWS

	IPlatformFile& platformFiles = FPlatformFileManager::Get().GetPlatformFile();
	FString localPath = FPlatformProcess::UserSettingsDir();
	localPath += "Monogram/Service/integrations/unreal";
	FString folderPath = FPaths::ProjectPluginsDir() + "Monogram/Resources/unreal";
#endif
	bool result = platformFiles.CopyDirectoryTree(*localPath, *folderPath, true);
}

void UMonogramEditorActions::Deinitialize()
{
}

void UMonogramEditorActions::PreChange(AActor* in) {
	if (!in) return;
	MonogramTransactionManager* aggrigator = MonogramTransactionManager::getAggrigator();
	if (aggrigator)
	{
		aggrigator->preAction(in);
	}
}

void UMonogramEditorActions::PostChange(AActor* in) {
	if (!in) return;
	MonogramTransactionManager* aggrigator = MonogramTransactionManager::getAggrigator();
	if (aggrigator)
	{
		aggrigator->postAction(in);
	}
}

void FMonogramEditorActionsModuleInterface::StartupModule()
{

}

void FMonogramEditorActionsModuleInterface::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
}


#undef LOCTEXT_NAMESPACE
IMPLEMENT_MODULE(FMonogramEditorActionsModuleInterface, MonogramEditorActions)
