//Copyright Monogram (Grant & Union) 2021
#include "MonogramTransactionAggrigator.h"
#include "Runtime/Core/Public/Async/Async.h"

MonogramTransactionManager* MonogramTransactionManager::SingletonInstance = nullptr;
FCriticalSection MonogramTransactionManager::SingletonMutex;

MonogramTransactionManager* MonogramTransactionManager::getAggrigator() {
	FScopeLock scopeLock(&SingletonMutex);
	if (SingletonInstance == nullptr) {
		SingletonInstance = new MonogramTransactionManager();
	}
	return SingletonInstance;
}

MonogramTransactionManager::MonogramTransactionManager() {
	this->runningThread = FRunnableThread::Create(&transThread, TEXT("TransactionTimer"), 0);
}

void MonogramTransactionManager::preAction(AActor* modifiedActor) {
	if (!openTransaction)
	{
		GEditor->BeginTransaction(FText::FromString("MONOGRAM CONTROL"));
		openTransaction = true;
	}

	AActor* Actor = modifiedActor;
	USceneComponent* SceneComponent = Actor->GetRootComponent();

	if (SceneComponent)
	{
		if (SceneComponent->HasAnyFlags(RF_DefaultSubObject))
		{
			SceneComponent->SetFlags(RF_Transactional);
		}
		SceneComponent->Modify(true);
		Actor->Modify(true);
	}

	skipTransaction = true;
	Actor->PreEditChange(NULL);
}

void MonogramTransactionManager::postAction(AActor* modifiedActor) {
	AActor* Actor = modifiedActor;
	USceneComponent* SceneComponent = Actor->GetRootComponent();
	Actor->PostEditMove(true);
	Actor->PostEditChange();
	SceneComponent->PostEditChange();
	GUnrealEd->UpdatePivotLocationForSelection();
	GUnrealEd->SetPivotMovedIndependently(false);
	SaveToTransactionBuffer(Actor, true);
	if (!GIsSavingPackage) {
		SnapshotTransactionBuffer(Actor);
	}
}

void TransactionThread::Stop()
{
	return;
}

bool TransactionThread::Init()
{
	return true;
}

uint32 TransactionThread::Run()
{
	MonogramTransactionManager* aggrigator = MonogramTransactionManager::getAggrigator();
	if (aggrigator == nullptr) return 0; //This shouldn't happen because the thread is started by the transaction manager constructor
		
	while (true)
	{
		FPlatformProcess::Sleep(0.2f);
		if (aggrigator->skipTransaction)
		{
			aggrigator->skipTransaction = false;
			continue;
		}

		if (aggrigator->openTransaction) {
			AsyncTask(ENamedThreads::GameThread, [=]()
			{
				if (aggrigator->openTransaction)
				{
					aggrigator->openTransaction = false;
					if (GEditor) {
						GEditor->EndTransaction();
					}
				}
			});
		}
	}
}