//Copyright Monogram (Grant & Union) 2021
#include "SequencerAction.h"

TWeakPtr<ISequencer> USequencerAction::Sequencer = nullptr;
FCriticalSection USequencerAction::SequencerMutex;
FThreadSafeBool USequencerAction::Initialized = false;

USequencerAction::USequencerAction() {
	if (USequencerAction::Initialized.AtomicSet(true) == false) {
		typedef TSharedRef<ISequencer> SequencerType;
		DECLARE_DELEGATE_OneParam(OnNewSequence, SequencerType)
		OnNewSequence onNewSequence;
		FSequencerCommands::Register();
		onNewSequence.BindLambda([&](TSharedRef<ISequencer> SequencerIn) {
			USequencerAction::SetSequencer(SequencerIn);
		});
		FModuleManager::GetModuleChecked<ISequencerModule>(TEXT("Sequencer")).RegisterOnSequencerCreated(onNewSequence);
	}
}

void USequencerAction::SetSequencer(TSharedPtr<ISequencer> SequencerPtr) {
	FScopeLock scopeLock(&USequencerAction::SequencerMutex);
	USequencerAction::Sequencer = TWeakPtr<ISequencer>(SequencerPtr);
}

void USequencerAction::Run(FName Command, Operation Operation, float Value) {
	AsyncTask(ENamedThreads::GameThread, [=]() {
		if (this->GetSequencer().IsValid()) {
			this->Handle(Command, Operation, Value);
		}
		else {
			UE_LOG(LogTemp, Warning, TEXT("Failed to run sequencer task '%s', sequencer not ready"), *this->Name().ToString());
		}
	});
}

TSharedPtr<ISequencer> USequencerAction::GetSequencer()
{
	FScopeLock scopeLock(&USequencerAction::SequencerMutex);
	if (USequencerAction::Sequencer.IsValid()) {
		return this->Sequencer.Pin();
	}
	else {
		return nullptr;
	}
}
